<#
* Copyright(c) 2006-2016 ThreatSTOP, Inc. All Rights Reserved
* NOTICE: All information contained herein is, and remains the property of ThreatSTOP, Inc. and its suppliers, if any. The intellectual and 
* technical concepts contained herein are proprietary to ThreatSTOP, Inc. and its suppliers and certain aspects thereof are protected by United 
* States Patent No. 8,533,822 and United States Patent No. 8,869,237, and are protected by trade secret or copyright law.
* US Government Entities: The ThreatSTOP service, software and documentation, as applicable, are "commercial computer software" and
* "commercial computer software documentation" developed exclusively at private expense by Threatstop, Inc. ("Threatstop"). Pursuant to FAR
* 12.212 or DFARS 227 7202 and their successors, as applicable, use, reproduction and disclosure of the software and documentation is
* governed by the terms of Threatstop's commercial agreement.
#>
#ErrID start: 10
#
# 1. load config values
# 2. create or update scheduled task for loading policies
# 3. create or update scheduled task for exporting logs
# 4. create or update scheduled task for uploading logs
#
############

#1. Load Config
$scriptPath = split-path -parent $MyInvocation.MyCommand.Definition
$iniPath = (get-item $scriptPath).parent.fullname
Get-Content "$iniPath\ThreatSTOP.ini" | foreach-object -begin {$h=@{}} -process { $k = [regex]::split($_,'='); if(($k[0].CompareTo("") -ne 0) -and ($k[0].StartsWith("[") -ne $True -and $k[0].StartsWith("#")  -ne $True)) { $h.Add($k[0].Trim(), $k[1].Trim()) } }

#2 create policy refresh task
$taskName = 'ThreatSTOP_policyrefresh'
$taskExists = Get-ScheduledTask | Where-Object {$_.TaskName -like $taskName }
if($taskExists) {
   Unregister-ScheduledTask -TaskName $taskName -Confirm:$false
}
try
{
	if ($h.UseDig -ne "true"){$applyscript="applyTSpolicy.ps1"} else {$applyscript="digTSZone.ps1"}
	$action = New-ScheduledTaskAction -Execute "$PsHome\Powershell.exe" -Argument "-NonInteractive -NoLogo -NoProfile -File `"$scriptPath\$applyscript`"" 
	$trigger = New-ScheduledTaskTrigger -Once -At '3AM' -RepetitionInterval (New-TimeSpan -Minutes $h.RefreshPolicyTimer)  -RandomDelay (New-TimeSpan -Minutes 5)
	Register-ScheduledTask -TaskName 'ThreatSTOP_policyrefresh' -Trigger $trigger -Action $action -description "ThreatSTOP DNS policy refresh" -User "NT AUTHORITY\SYSTEM" -RunLevel 1
	Set-ScheduledTask 'ThreatSTOP_policyrefresh' -Trigger $trigger
	Start-ScheduledTask 'ThreatSTOP_policyrefresh'
	Write-Host "Policy update scheduled task created."
}
catch {
	$errmsg = 'Policy refresh task creation error: ' + $_.Exception.Message
	Write-EventLog -LogName Application -Source 'ThreatSTOP' -EntryType Warning -EventId 10 -Message $errmsg
}

#3 create export log task
$taskName = 'ThreatSTOP_exportDNSlog'
$taskExists = Get-ScheduledTask | Where-Object {$_.TaskName -like $taskName }
if($taskExists) {
   Unregister-ScheduledTask -TaskName $taskName -Confirm:$false
}
try
{
	$action = New-ScheduledTaskAction -Execute "$PsHome\Powershell.exe" -Argument "-NonInteractive -NoLogo -NoProfile -File `"$scriptPath\exportDNSlog.ps1`"" 
	$trigger = New-ScheduledTaskTrigger -Once -At '3AM' -RepetitionInterval (New-TimeSpan -Minutes $h.LogExportTimer) -RandomDelay (New-TimeSpan -Minutes 5)
	Register-ScheduledTask -TaskName 'ThreatSTOP_exportDNSlog' -Trigger $trigger -Action $action -description "ThreatSTOP DNS log export" -User "NT AUTHORITY\SYSTEM" -RunLevel 1
	Set-ScheduledTask 'ThreatSTOP_exportDNSlog' -Trigger $trigger
	Start-ScheduledTask 'ThreatSTOP_exportDNSlog'
	Write-Host "DNS Policy export log scheduled task created."
}
catch {
	$errmsg = 'Export DNS Policy log task creation error: ' + $_.Exception.Message
	Write-EventLog -LogName Application -Source 'ThreatSTOP' -EntryType Warning -EventId 11 -Message $errmsg
}

#4. Create upload log task
$taskName = 'ThreatSTOP_uploadDNSlog'
$taskExists = Get-ScheduledTask | Where-Object {$_.TaskName -like $taskName }
if($taskExists) {
   Unregister-ScheduledTask -TaskName $taskName -Confirm:$false
}

if ($h.UploadLogs -eq "true")
{
	try
	{
		$action = New-ScheduledTaskAction -Execute "$PsHome\Powershell.exe" -Argument "-NonInteractive -NoLogo -NoProfile -File `"$scriptPath\uploadDNSlog.ps1`"" 
		$trigger = New-ScheduledTaskTrigger -Once -At '3AM' -RepetitionInterval (New-TimeSpan -Minutes $h.LogUploadTimer) -RandomDelay (New-TimeSpan -Minutes 5)
		Register-ScheduledTask -TaskName 'ThreatSTOP_uploadDNSlog' -Trigger $trigger -Action $action -description "ThreatSTOP DNS log upload" -User "NT AUTHORITY\SYSTEM" -RunLevel 1
		Set-ScheduledTask 'ThreatSTOP_uploadDNSlog' -Trigger $trigger
		Start-ScheduledTask 'ThreatSTOP_uploadDNSlog'
		Write-Host "DNS Policy upload log scheduled task created."
	}
	catch {
		$errmsg = 'Upload DNS Policy log task creation error: ' + $_.Exception.Message
		Write-EventLog -LogName Application -Source 'ThreatSTOP' -EntryType Warning -EventId 12 -Message $errmsg
	}
}
